Package: DropSeq.utilities
Type: Package
Title: Generic Drop-seq utilities
Version: 3.0.2
Date: 2024-07-15
Imports: data.table
Author: Alec Wysoker
Maintainer: McCarroll Lab <dropseq@gmail.com>
Description: Grab-bag of generic functions for handling strings, files, errors.
License: MIT + file LICENSE
RoxygenNote: 7.3.1
Encoding: UTF-8
