Package: DropSeq.dropulation
Type: Package
Title: Tools for analyzing Dropulation data
Version: 3.0.2
Date: 2024-07-15
Imports: graphics, utils, stats, data.table,RColorBrewer,vegan, gridExtra,
    ggplot2,ggrepel,purrr,stringr,scales
Suggests: DropSeq.utilities
Remotes: github::broadinstitute/Drop-seq/src/R/packages/DropSeq.utilities
Author: Jim Nemesh
Maintainer: McCarroll Lab <dropseq@gmail.com>
Description: Dropulation and Dropulation census functions.
License: MIT + file LICENSE
RoxygenNote: 7.3.1
Encoding: UTF-8
