
# DropSeq.utilities

<!-- badges: start -->
<!-- badges: end -->

DropSeq.utilities contains trivial functions to support other DropSeq R packages.

## Installation

The preferred installation method is to go to the latest release in
[Releases](https://github.com/broadinstitute/Drop-seq/releases) and follow the
instructions there.

It is not recommended to install the latest version of DropSeq.utilities from
[GitHub](https://github.com/), because it is not necessarily stable, but if you choose to do so:

``` r
# install.packages("devtools")
devtools::install_github("broadinstitute/Drop-seq", subdir="src/R/packages/DropSeq.utilities")
```

